package org.vraptor.config;

import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.List;

import org.apache.log4j.Logger;
import org.vraptor.config.xml.BasePackageConverter;
import org.vraptor.config.xml.ComponentConfigConverter;
import org.vraptor.config.xml.ConverterConfigConverter;
import org.vraptor.config.xml.FactoryConfigConverter;
import org.vraptor.config.xml.PluginConfigConverter;
import org.vraptor.config.xml.RegerViewManagerConfigConverter;
import org.vraptor.webapp.WebApplication;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Reads the xml configuration file
 * 
 * @author Guilherme Silveira
 */
public class XMLConfigurationLoader {

	private static final Logger LOG = Logger.getLogger(XMLConfigurationLoader.class);

	private XStream xstream;

	/**
	 * Constructs an xstream reader
	 * 
	 */
	public XMLConfigurationLoader() {
		xstream = new XStream(new DomDriver());
		xstream.registerConverter(new ComponentConfigConverter());
		xstream.registerConverter(new FactoryConfigConverter());
		xstream.registerConverter(new ConverterConfigConverter());
		xstream.registerConverter(new PluginConfigConverter());
		xstream.registerConverter(new RegerViewManagerConfigConverter());
		xstream.registerConverter(new BasePackageConverter());
		xstream.alias("factory", FactoryConfig.class);
		xstream.alias("component", ComponentConfig.class);
		xstream.alias("plugin", PluginConfig.class);
		xstream.alias("converter", ConverterConfig.class);
		xstream.alias("package", BasePackageConfig.class);
		xstream.alias("regex-view-manager", RegexViewManagerConfig.class);
	}

	/**
	 * Reads a file and uses it to register vraptor's data in the controller
	 * 
	 * @param stream
	 *            stream
	 * @param controller
	 *            controller
	 * @throws ConfigException
	 *             some configuration error
	 * @throws IOException
	 *             some input exception occurred
	 */
	@SuppressWarnings("unchecked")
	public void readComponents(InputStream stream, WebApplication application) throws ConfigException, IOException {
		InputStreamReader reader = new InputStreamReader(stream);
		XStream xstream = getXStream();
		ObjectInputStream in = xstream.createObjectInputStream(reader);
	
		while (true) {
			try {
				ConfigItem item = (ConfigItem) in.readObject();
				if (item.isComponent()) {
					item.register(application);
				}
			} catch (EOFException e) {
				LOG.debug("finished reading file");
				break;
			} catch (ClassNotFoundException e) {
				throw new ConfigException(e.getMessage(), e);
			}
		}
		in.close();
		reader.close();
	}

	/**
	 * Reads all managers
	 * 
	 * @param stream
	 *            stream
	 * @param controller
	 *            controller
	 * @throws ConfigException
	 *             configuration exceptions
	 * @throws IOException
	 *             ioexceptions
	 */
	@SuppressWarnings("unchecked")
	public void readManagers(InputStream stream, WebApplication application) throws ConfigException, IOException {
		InputStreamReader reader = new InputStreamReader(stream);
		XStream xstream = getXStream();
		ObjectInputStream in = xstream.createObjectInputStream(reader);
		while (true) {
			try {
				ConfigItem item = (ConfigItem) in.readObject();
				if (item.isManager()) {
					item.register(application);
				}
			} catch (EOFException e) {
				LOG.debug("finished reading file");
				break;
			} catch (ClassNotFoundException e) {
				throw new ConfigException(e.getMessage(), e);
			}
		}
		in.close();
		reader.close();
	}

	/**
	 * Writes all items in a config output
	 * 
	 * @param stream
	 *            stream output
	 * @param items
	 *            items
	 * @throws IOException
	 *             output exception
	 */
	@SuppressWarnings("unchecked")
	public void write(OutputStream stream, List<ConfigItem> items) throws IOException {
		PrintWriter writer = new PrintWriter(stream);
		XStream xstream = getXStream();
		ObjectOutputStream out = xstream.createObjectOutputStream(writer, "vraptor");
		for (ConfigItem item : items) {
			out.writeObject(item);
		}
		out.flush();
		out.close();
		writer.flush();
		writer.close();
	}

	/**
	 * Returns the xstream instance
	 * 
	 * @return the instance
	 */
	private XStream getXStream() {
		return xstream;
	}

}
