package org.vraptor.core;

import java.io.IOException;
import java.util.LinkedList;

import javax.servlet.ServletException;

import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.LogicRequest;
import org.vraptor.RedirectException;
import org.vraptor.component.ComponentTypeFactory;
import org.vraptor.component.ComponentInstantiationException;
import org.vraptor.component.ComponentType;
import org.vraptor.interceptor.InterceptorClass;
import org.vraptor.interceptor.InterceptorDealer;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.reflection.GettingException;
import org.vraptor.reflection.SettingException;
import org.vraptor.url.SimpleLogicRequest;
import org.vraptor.view.ViewException;

/**
 * The base implementation of logic flow.
 * 
 * @author Guilherme Silveira
 * 
 */
public class BasicLogicFlow implements LogicFlow {

	private static final Logger LOG = Logger.getLogger(BasicLogicFlow.class);

	private final InterceptorDealer interceptorDealer;

	private final LinkedList<Interceptor> interceptors = new LinkedList<Interceptor>();

	private final LinkedList<InterceptorClass> interceptorsClass;

	private Interceptor lastInterceptor;

	private InterceptorClass lastInterceptorClass;

	private final SimpleLogicRequest request;

	public BasicLogicFlow(ComponentType componentClass,
			VRaptorExecution engine, Interceptor[] before, Interceptor[] after)
			throws InterceptorInstantiationException {
		LOG.debug("Creating new basic logic flow");
		this.interceptorsClass = new LinkedList<InterceptorClass>();
		this.request = engine.getRequest();
		this.interceptorDealer = new InterceptorDealer(engine.getController()
				.getIntrospector());
		registerInterceptors(before);
		for (InterceptorClass clazz : componentClass.getInterceptors()) {
			try {
				Interceptor i = clazz.newInstance();
				LOG.debug("Adding interceptor " + i.getClass().getName());
				interceptors.addLast(i);
				interceptorsClass.addLast(clazz);
			} catch (ComponentInstantiationException e) {
				throw new InterceptorInstantiationException(e.getMessage(), e);
			}
		}
		registerInterceptors(after);
	}

	private void registerInterceptors(Interceptor[] toAdd) {
		for (Interceptor interceptor : toAdd) {
			LOG.debug("Adding interceptor " + interceptor.getClass().getName());
			interceptors.addLast(interceptor);
			InterceptorClass clazz = ComponentTypeFactory.INTERCEPTOR_LIST_FACTORY
					.getInterceptorClass(interceptor.getClass());
			interceptorsClass.addLast(clazz);
		}
	}

	/**
	 * Executes the next step in the flow.
	 */
	public void execute() throws ViewException, LogicException {
		LOG.debug("Calling execute on flow");
		if (lastInterceptor != null) {
			try {
				interceptorDealer.outject(lastInterceptor,
						lastInterceptorClass, request);
			} catch (GettingException e) {
				throw new LogicException(e.getMessage(), e);
			}
		}
		lastInterceptor = interceptors.removeFirst();
		lastInterceptorClass = interceptorsClass.removeFirst();
		try {
			interceptorDealer.inject(lastInterceptor, lastInterceptorClass,
					request);
		} catch (ComponentInstantiationException e) {
			throw new LogicException(e.getMessage(), e);
		} catch (SettingException e) {
			throw new LogicException(e.getMessage(), e);
		}
		lastInterceptor.intercept(this);
	}

	public void redirect(String url) throws RedirectException {
		try {
			request.getRequest().getRequestDispatcher(url).forward(
					request.getRequest(), request.getResponse());
		} catch (ServletException e) {
			if (e.getRootCause() == null) {
				throw new RedirectException(e.getMessage(), e);
			}
			throw new RedirectException(e.getRootCause().getMessage(), e
					.getRootCause());
		} catch (IOException e) {
			throw new RedirectException(e.getMessage(), e);
		}
	}

	public LogicRequest getLogicRequest() {
		return this.request;
	}

}
