package org.vraptor.interceptor;

import java.io.File;
import java.io.IOException;
import java.util.List;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.log4j.Logger;
import org.vraptor.Interceptor;
import org.vraptor.LogicException;
import org.vraptor.LogicFlow;
import org.vraptor.annotations.In;
import org.vraptor.http.VRaptorRequest;
import org.vraptor.scope.ScopeType;
import org.vraptor.view.ViewException;

/**
 * Interceptor capable of parsing the input stream.
 * 
 * @author Guilherme Silveira
 * @author Paulo Silveira
 */
public class MultipartRequestInterceptor implements Interceptor {
	private Logger logger = Logger.getLogger(MultipartRequestInterceptor.class);

	@In(scope = ScopeType.LOGIC)
	private VRaptorRequest request;

	private final File temporaryDirectory;

	public MultipartRequestInterceptor() throws IOException {
		// this directory must be configurable through the properties
		this.temporaryDirectory = File.createTempFile("raptor.", ".upload")
				.getParentFile();
	}

	@SuppressWarnings("unchecked")
	public void intercept(LogicFlow flow) throws LogicException, ViewException {

		if (!ServletFileUpload.isMultipartContent(request)) {
			flow.execute();
			return;
		}

		this.logger.debug("Trying to parse multipart request.");

		// Create a factory for disk-based file items
		DiskFileItemFactory factory = new DiskFileItemFactory(4096 * 16,
				this.temporaryDirectory);
		this.logger.debug("Using repository [" + factory.getRepository() + "]");

		// Create a new file upload handler
		ServletFileUpload upload = new ServletFileUpload(factory);

		// TODO: variables in raptor.properties
		upload.setSizeMax(2000000);

		List<FileItem> fileItems;

		// assume we know there are two files. The first file is a small
		// text file, the second is unknown and is written to a file on
		// the server
		try {
			fileItems = upload.parseRequest(request);
		} catch (FileUploadException e) {
			this.logger
					.warn("There was some problem parsing this multipart request, or someone is not sending a RFC1867 compatible multipart request.",e);
			flow.execute();
			return;
		}

		this.logger
				.debug("Found ["
						+ fileItems.size()
						+ "] attributes in the multipart form submission. Parsing them.");

		for (FileItem item : fileItems) {
			if (item.isFormField()) {
				request.overwriteParameters(item.getFieldName(), item
						.getString());
			} else {
				if (!item.getName().trim().equals("")) {
					try {
						File file = File.createTempFile("raptor.", ".upload");
						file.deleteOnExit();
						item.write(file);
						UploadedFileInformation fileInformation = new BasicUploadedFileInformation(
								file, item.getName(), item.getContentType());
						request.setAttribute(item.getFieldName(),
								fileInformation);
						this.logger.info("Uploaded file: " + item.getFieldName() + " with " + fileInformation);
					} catch (Exception e) {
						logger
								.error("Nasty uploaded file " + item.getName(),
										e);
					}
				} else {
					this.logger.info("A file field was empy: "
							+ item.getFieldName());
				}
			}
		}

		flow.execute();

		// should we delete the temporary files afterwards or onExit as done by
		// now?
		// maybe also a config in .properties

	}
}
