package org.vraptor.view;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.LogicRequest;
import org.vraptor.introspector.ExpressionEvaluationException;
import org.vraptor.introspector.ExpressionEvaluator;

/**
 * Overriden view manager checks if the view was overriden and returns it.
 * Otherwise it returns the default view that its internal view manager would
 * return.
 * 
 * @author Guilherme Silveira
 */
public class OverridenViewManager implements ViewManager {

	private final ExpressionEvaluator eval = new ExpressionEvaluator();

	private static final Logger LOG = Logger.getLogger(OverridenViewManager.class);

	private final Map<String, String> views = new HashMap<String, String>();

	private final ViewManager internalManager;

	/**
	 * Creates the overriden view manager with a default view manager
	 * 
	 * @param viewManager
	 *            the default view manager
	 */
	public OverridenViewManager(ViewManager viewManager) {
		this.internalManager = viewManager;
	}

	/**
	 * Register a new view
	 * 
	 * @param key
	 *            key
	 * @param value
	 *            value
	 */
	public void overrideView(String key, String value) {
		LOG.info(String.format("Overriding view %s --> %s", key, value));
		this.views.put(key, value);
	}

	public void forward(LogicRequest logicRequest, String result, HttpServletRequest request,
			HttpServletResponse response) throws ViewException {
		// if it is overriden, overrides the forward
		String key = logicRequest.getComponentName() + "." + logicRequest.getLogicName() + "." + result;
		try {
			result = eval.parseExpression(result, logicRequest);
		} catch (ExpressionEvaluationException e) {
			throw new ViewException(e);
		}
		if (this.views.containsKey(key)) {
			if (LOG.isDebugEnabled()) {
				LOG.debug("overriden view found " + key);
			}
			String forward = this.views.get(key);
			if (forward.startsWith("redirect:")) {
				redirect(logicRequest, response, forward.substring(9));
			} else {
				internalManager.directForward(logicRequest, result, request, response, forward);
			}
		} else {
			internalManager.forward(logicRequest, result, request, response);
		}
	}

	public void directForward(LogicRequest logicRequest, String result, HttpServletRequest request,
			HttpServletResponse response, String forwardUrl) throws ViewException {
		internalManager.directForward(logicRequest, result, request, response, forwardUrl);
	}

	public void redirect(LogicRequest request, HttpServletResponse response, String result) throws ViewException {
		try {
			internalManager.redirect(request, response, eval.parseExpression(result, request));
		} catch (ExpressionEvaluationException e) {
			throw new ViewException(e);
		}
	}

}
